  
/*
  
  By: bldrkamal
  
  Hardware hookup:
  esp32 --------------- MMA8452Q Breakout
    3.3V  ---------------     3.3V
    GND   ---------------     GND
  SDA (D21) -----------------  SDA
  SCL (D22) ---------------    SCL
 
*/ 
#include <WiFi.h>
#include <PubSubClient.h>

#include <Wire.h>                 // Must include Wire library for I2C
#include "SparkFun_MMA8452Q.h"  


// Change the credentials below, so your ESP8266 connects to your router
const char* ssid = "HUAWEI-8E4BAC";
const char* wifi_password = "12345678";

// Change the variable to your Raspberry Pi IP address, so it connects to your MQTT broker
const char* mqtt_server = "192.168.8.101";

// Initializes the espClient. You should change the espClient name if you have multiple ESPs running in your home automation system
WiFiClient wifiClient;
PubSubClient client(wifiClient);


// LED - GPIO 23 on ESP-32 devkit
const int led = 23;

// Initialize accelerometer sensor.
MMA8452Q accel;

// Timers auxiliar variables
long now = millis();
long lastMeasure = 0;

// Don't change the function below. This functions connects your ESP32 to your router
void setup_wifi() {
  delay(10);
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, wifi_password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.print("WiFi connected - ESP IP address: ");
  Serial.println(WiFi.localIP());
}

// This functions is executed when some device publishes a message to a topic that your ESP32 is subscribed to
// Change the function below to add logic to your program, so when a device publishes a message to a topic that 
// your ESP32 is subscribed you can actually do something
void callback(String topic, byte* message, unsigned int length) {
  Serial.print("Message arrived on topic: ");
  Serial.print(topic);
  Serial.print(". Message: ");
  String messageTemp;
  
  for (int i = 0; i < length; i++) {
    Serial.print((char)message[i]);
    messageTemp += (char)message[i];
  }
  Serial.println();

  // Feel free to add more if statements to control more GPIOs with MQTT

  // If a message is received on the topic test, you check if the message is either on or off. Turns the led GPIO according to the message
  if(topic=="test"){
      Serial.print("Changing Room lamp to ");
      if(messageTemp == "on"){
        digitalWrite(led, HIGH);
        Serial.print("On");
        
        uint32_t period = 5 * 6000L;       

      for( uint32_t tStart = millis();  (millis()-tStart) < period;  ){
   now = millis();
  
  if (now - lastMeasure > 100) {
    lastMeasure = now;
    
    accel.read();
    float value=accel.cz;
    // Check if any reads failed and exit early (to try again).
    if (isnan(value)) {
      Serial.println("Failed to read from accel sensor!");
      return;
    }

   
    
    static char accel[7];
    dtostrf(value, 6, 2, accel);

    
    client.publish("accel", accel);
    
    Serial.print("accelerometer: ");
    Serial.print(value);
    
    Serial.println();
    
  }
            }

       digitalWrite(led,LOW);
       Serial.print("off");     
     }
     
  }
  Serial.println();
}

// This functions reconnects your ESP32 to your MQTT broker
// Change the function below if you want to subscribe to more topics with your ESP32 
void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Attempt to connect
    
    if (client.connect("esp32")) {
      Serial.println("connected");  
      // Subscribe or resubscribe to a topic
      // You can subscribe to more topics (to control more LEDs in this example)
      client.subscribe("test");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}

// The setup function sets your ESP GPIOs to Outputs, starts the serial communication at a baud rate of 115200
// Sets your mqtt broker and sets the callback function
// The callback function is what receives messages and actually controls the LEDs
void setup() {
  pinMode(led, OUTPUT);
  

  Wire.begin();

  if (accel.begin() == false) {
    //Serial.println("Not Connected. Please check connections and read the hookup guide.");
    while (1);}
  
  Serial.begin(115200);
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);

}

// For this project, you don't need to change anything in the loop function. Basically it ensures that you ESP is connected to your broker
void loop() {

  if (!client.connected()) {
    reconnect();
  }
  if(!client.loop())
    client.connect("esp32");


} 
